import * as beet from '@metaplex-foundation/beet';
import { CANDY_MACHINE_HIDDEN_SECTION } from '../constants.mjs';
import { deserializeFeatureFlags } from '../../../types/FeatureFlags.mjs';
import assert from '../../../utils/assert.mjs';
import { removeEmptyChars } from '../../../utils/common.mjs';
import { toBigNumber } from '../../../types/BigNumber.mjs';

/** @internal */

/** @internal */
const deserializeCandyMachineHiddenSection = (buffer, itemsAvailable, itemsRemaining, configLineSettings, offset = 0) => {
  // Items loaded.
  const itemsLoaded = beet.u32.read(buffer, offset);
  offset += 4; // Raw config lines.

  const {
    nameLength
  } = configLineSettings;
  const {
    uriLength
  } = configLineSettings;
  const configLineSize = nameLength + uriLength;
  const configLinesSize = configLineSize * itemsAvailable;
  const rawConfigLines = buffer.slice(offset, offset + configLinesSize);
  offset += configLinesSize; // Items loaded map.

  const [itemsLoadedMap] = deserializeFeatureFlags(buffer, itemsAvailable, offset, false);
  const itemsLoadedMapSize = Math.floor(itemsAvailable / 8) + 1;
  offset += itemsLoadedMapSize; // Items left to mint for random order only.

  const itemsLeftToMint = beet.uniformFixedSizeArray(beet.u32, itemsAvailable).read(buffer, offset).slice(0, itemsRemaining); // Helper function to figure out if an item has been minted.

  const itemsMinted = itemsAvailable - itemsRemaining;

  const isMinted = index => configLineSettings.isSequential ? index < itemsMinted : !itemsLeftToMint.includes(index); // Parse config lines.


  const items = [];
  itemsLoadedMap.forEach((loaded, index) => {
    if (!loaded) return;
    const namePosition = index * configLineSize;
    const uriPosition = namePosition + nameLength;
    const name = rawConfigLines.slice(namePosition, namePosition + nameLength).toString('utf8');
    const uri = rawConfigLines.slice(uriPosition, uriPosition + uriLength).toString('utf8');
    const prefixName = replaceCandyMachineItemPattern(configLineSettings.prefixName, index);
    const prefixUri = replaceCandyMachineItemPattern(configLineSettings.prefixUri, index);
    items.push({
      index,
      minted: isMinted(index),
      name: prefixName + removeEmptyChars(name),
      uri: prefixUri + removeEmptyChars(uri)
    });
  });
  return {
    itemsLoaded,
    items,
    itemsLoadedMap,
    itemsLeftToMint
  };
};
/** @internal */

const getCandyMachineSize = data => {
  if (data.hiddenSettings) {
    return CANDY_MACHINE_HIDDEN_SECTION;
  } // This should not happen as the candy machine input type
  // ensures exactly on of them is provided.


  assert(!!data.configLineSettings, 'No config line settings nor hidden settings were provided. ' + 'Please provide one of them.');
  const itemsAvailable = toBigNumber(data.itemsAvailable).toNumber();
  const configLineSize = data.configLineSettings.nameLength + data.configLineSettings.uriLength;
  return Math.ceil(CANDY_MACHINE_HIDDEN_SECTION + // Number of currently items inserted.
  4 + // Config line data.
  itemsAvailable * configLineSize + ( // Bit mask to keep track of which ConfigLines have been added.
  4 + Math.floor(itemsAvailable / 8) + 1) + ( // Mint indices.
  4 + itemsAvailable * 4));
};
/** @internal */

const replaceCandyMachineItemPattern = (value, index) => {
  return value.replace('$ID+1$', `${index + 1}`).replace('$ID$', `${index}`);
};

export { deserializeCandyMachineHiddenSection, getCandyMachineSize, replaceCandyMachineItemPattern };
//# sourceMappingURL=CandyMachineHiddenSection.mjs.map
